@@ -12,12 +12,11 @@ import marshal
 import os
 import struct
 import sys
-
 from importlib.machinery import ModuleSpec
 from types import CodeType, ModuleType
 from typing import Any
 
-from coverage.exceptions import CoverageException, _ExceptionDuringRun, NoCode, NoSource
+from coverage.exceptions import CoverageException, NoCode, NoSource, _ExceptionDuringRun
 from coverage.files import canonical_filename, python_reported_file
 from coverage.misc import isolate_module
 from coverage.python import get_python_source
@@ -27,11 +26,13 @@ os = isolate_module(os)
 
 PYC_MAGIC_NUMBER = importlib.util.MAGIC_NUMBER
 
+
 class DummyLoader:
     """A shim for the pep302 __loader__, emulating pkgutil.ImpLoader.
 
     Currently only implements the .fullname attribute
     """
+
     def __init__(self, fullname: str, *_args: Any) -> None:
         self.fullname = fullname
 
@@ -57,8 +58,8 @@ def find_module(
         spec = importlib.util.find_spec(mod_main)
         if not spec:
             raise NoSource(
-                f"No module named {mod_main}; " +
-                f"{modulename!r} is a package and cannot be directly executed",
+                f"No module named {mod_main}; "
+                + f"{modulename!r} is a package and cannot be directly executed",
             )
         pathname = spec.origin
         packagename = spec.name
@@ -72,6 +73,7 @@ class PyRunner:
     This is meant to emulate real Python execution as closely as possible.
 
     """
+
     def __init__(self, args: list[str], as_module: bool = False) -> None:
         self.args = args
         self.as_module = as_module
@@ -89,7 +91,10 @@ class PyRunner:
         This needs to happen before any importing, and without importing anything.
         """
         path0: str | None
-        if self.as_module:
+        if getattr(sys.flags, "safe_path", False):
+            # See https://docs.python.org/3/using/cmdline.html#cmdoption-P
+            path0 = None
+        elif self.as_module:
             path0 = os.getcwd()
         elif os.path.isdir(self.arg0):
             # Running a directory means running the __main__.py file in that
@@ -142,7 +147,7 @@ class PyRunner:
             # Running a directory means running the __main__.py file in that
             # directory.
             for ext in [".py", ".pyc", ".pyo"]:
-                try_filename = os.path.join(self.arg0, "__main__" + ext)
+                try_filename = os.path.join(self.arg0, f"__main__{ext}")
                 # 3.8.10 changed how files are reported when running a
                 # directory.
                 try_filename = os.path.abspath(try_filename)
@@ -177,11 +182,11 @@ class PyRunner:
             main_mod.__file__ = main_mod.__file__[:-1]
         if self.package is not None:
             main_mod.__package__ = self.package
-        main_mod.__loader__ = self.loader   # type: ignore[assignment]
+        main_mod.__loader__ = self.loader  # type: ignore[assignment]
         if self.spec is not None:
             main_mod.__spec__ = self.spec
 
-        main_mod.__builtins__ = sys.modules["builtins"]     # type: ignore[attr-defined]
+        main_mod.__builtins__ = sys.modules["builtins"]  # type: ignore[attr-defined]
 
         sys.modules["__main__"] = main_mod
 
@@ -206,7 +211,7 @@ class PyRunner:
         cwd = os.getcwd()
         try:
             exec(code, main_mod.__dict__)
-        except SystemExit:                          # pylint: disable=try-except-raise
+        except SystemExit:  # pylint: disable=try-except-raise
             # The user called sys.exit().  Just pass it along to the upper
             # layers, where it will be handled.
             raise
@@ -232,7 +237,7 @@ class PyRunner:
                 assert err.__traceback__ is not None
                 err.__traceback__ = err.__traceback__.tb_next
                 sys.excepthook(typ, err, tb.tb_next)
-            except SystemExit:                      # pylint: disable=try-except-raise
+            except SystemExit:  # pylint: disable=try-except-raise
                 raise
             except Exception as exc:
                 # Getting the output right in the case of excepthook
@@ -311,11 +316,11 @@ def make_code_from_pyc(filename: str) -> CodeType:
         flags = struct.unpack("<L", fpyc.read(4))[0]
         hash_based = flags & 0x01
         if hash_based:
-            fpyc.read(8)    # Skip the hash.
+            fpyc.read(8)  # Skip the hash.
         else:
             # Skip the junk in the header that we don't need.
-            fpyc.read(4)    # Skip the moddate.
-            fpyc.read(4)    # Skip the size.
+            fpyc.read(4)  # Skip the moddate.
+            fpyc.read(4)  # Skip the size.
 
         # The rest of the file is the code object we want.
         code = marshal.load(fpyc)
