@@ -8,7 +8,7 @@ from __future__ import annotations
 
 # version_info: same semantics as sys.version_info.
 # _dev: the .devN suffix if any.
-version_info = (7, 0, 5, "final", 0)
+version_info = (7, 10, 5, "final", 0)
 _dev = 0
 
 
@@ -21,10 +21,10 @@ def _make_version(
     dev: int = 0,
 ) -> str:
     """Create a readable version string from version_info tuple components."""
-    assert releaselevel in ['alpha', 'beta', 'candidate', 'final']
+    assert releaselevel in ["alpha", "beta", "candidate", "final"]
     version = "%d.%d.%d" % (major, minor, micro)
-    if releaselevel != 'final':
-        short = {'alpha': 'a', 'beta': 'b', 'candidate': 'rc'}[releaselevel]
+    if releaselevel != "final":
+        short = {"alpha": "a", "beta": "b", "candidate": "rc"}[releaselevel]
         version += f"{short}{serial}"
     if dev != 0:
         version += f".dev{dev}"
@@ -40,11 +40,9 @@ def _make_url(
     dev: int = 0,
 ) -> str:
     """Make the URL people should start at for this version of coverage.py."""
-    url = "https://coverage.readthedocs.io"
-    if releaselevel != "final" or dev != 0:
-        # For pre-releases, use a version-specific URL.
-        url += "/en/" + _make_version(major, minor, micro, releaselevel, serial, dev)
-    return url
+    return "https://coverage.readthedocs.io/en/" + _make_version(
+        major, minor, micro, releaselevel, serial, dev
+    )
 
 
 __version__ = _make_version(*version_info, _dev)
